@@ -225,7 +225,10 @@ static struct ipapwd_krbcfg *ipapwd_getConfig(void)
         goto free_and_error;
     }
     config->passsync_mgrs =
-            slapi_entry_attr_get_charray(config_entry, "passSyncManagersDNs");
+        slapi_entry_attr_get_charray(config_entry, "passSyncManagersDNs");
+    config->sysacct_mgrs =
+        slapi_entry_attr_get_charray(config_entry, "SysAcctManagersDNs");
+
     /* now add Directory Manager, it is always added by default */
     tmpstr = slapi_ch_strdup("cn=Directory Manager");
     slapi_ch_array_add(&config->passsync_mgrs, tmpstr);
@@ -233,8 +236,6 @@ static struct ipapwd_krbcfg *ipapwd_getConfig(void)
         LOG_OOM();
         goto free_and_error;
     }
-    for (i = 0; config->passsync_mgrs[i]; i++) /* count */ ;
-    config->num_passsync_mgrs = i;
 
     slapi_entry_free(config_entry);
 
@@ -289,6 +290,7 @@ free_and_error:
         free(config->pref_encsalts);
         free(config->supp_encsalts);
         slapi_ch_array_free(config->passsync_mgrs);
+        slapi_ch_array_free(config->sysacct_mgrs);
         free(config);
     }
     slapi_entry_free(config_entry);
@@ -333,7 +335,9 @@ int ipapwd_getPolicy(const char *dn,
                       "krbPwdMinDiffChars", "krbPwdMinLength",
                       "krbPwdHistoryLength", "ipaPwdMaxRepeat",
                       "ipaPwdMaxSequence", "ipaPwdDictCheck",
-                      "ipaPwdUserCheck", NULL};
+                      "ipaPwdUserCheck", "ipapwdDcredit",
+                      "ipaPwdUcredit", "ipaPwdLcredit",
+                      "ipaPwdOcredit", NULL};
     Slapi_Entry **es = NULL;
     Slapi_Entry *pe = NULL;
     int ret, res, scope;
@@ -418,6 +422,10 @@ int ipapwd_getPolicy(const char *dn,
     policy->max_sequence = slapi_entry_attr_get_int(pe, "ipaPwdMaxSequence");
     policy->dictcheck = slapi_entry_attr_get_bool(pe, "ipaPwdDictCheck");
     policy->usercheck = slapi_entry_attr_get_bool(pe, "ipaPwdUserCheck");
+    policy->dcredit = slapi_entry_attr_get_int(pe, "ipaPwdDcredit");
+    policy->ucredit = slapi_entry_attr_get_int(pe, "ipaPwdUcredit");
+    policy->lcredit = slapi_entry_attr_get_int(pe, "ipaPwdLcredit");
+    policy->ocredit = slapi_entry_attr_get_int(pe, "ipaPwdOcredit");
 
     ret = 0;
 
@@ -608,6 +616,12 @@ int ipapwd_CheckPolicy(struct ipapwd_data *data)
 
     switch(data->changetype) {
         case IPA_CHANGETYPE_NORMAL:
+        case IPA_CHANGETYPE_SYSACCT:
+            /*
+             * Treat a system account-initiated password change as the user's
+             * initiated one as well, to force password quality checks on them.
+             */
+
             /* Find the entry with the password policy */
             ret = ipapwd_getPolicy(data->dn, data->target, &pol);
             if (ret) {
@@ -1137,4 +1151,3 @@ int ipapwd_check_max_pwd_len(size_t len, char **errMesg) {
     }
     return 0;
 }
-
