@@ -70,7 +70,7 @@ class CertUpdate(admintool.AdminTool):
             raise
         finally:
             if old_krb5ccname is None:
-                del os.environ['KRB5CCNAME']
+                os.environ.pop('KRB5CCNAME', None)
             else:
                 os.environ['KRB5CCNAME'] = old_krb5ccname
 
@@ -276,7 +276,7 @@ def update_db(path, certs):
     for name, flags in db.list_certs():
         if flags.ca:
             db.delete_cert(name)
-    for cert, nickname, trusted, eku in certs:
+    for cert, nickname, trusted, eku, _serial in certs:
         trust_flags = certstore.key_policy_to_trust_flags(trusted, True, eku)
         try:
             db.add_cert(cert, nickname, trust_flags)
