@@ -25,20 +25,19 @@ All constants centralised in one file.
 import os
 import string
 import uuid
-import warnings
-
-warnings.filterwarnings(
-    "ignore",
-    "TripleDES has been moved to "
-    "cryptography.hazmat.decrepit.ciphers.algorithms.TripleDES and "
-    "will be removed from this module in 48.0.0",
-    category=UserWarning)
 
 from ipaplatform.constants import constants as _constants
 from ipapython.dn import DN
 from ipapython.fqdn import gethostfqdn
 from ipapython.version import VERSION, API_VERSION
-from cryptography.hazmat.primitives.ciphers import algorithms, modes
+from cryptography.hazmat.primitives.ciphers import modes
+try:
+    # cryptography>=43.0.0
+    from cryptography.hazmat.decrepit.ciphers.algorithms import TripleDES
+except ImportError:
+    # will be removed from this module in cryptography 48.0.0
+    from cryptography.hazmat.primitives.ciphers.algorithms import TripleDES
+
 from cryptography.hazmat.backends.openssl.backend import backend
 
 
@@ -64,6 +63,8 @@ TLS_VERSION_MAXIMAL = "tls1.3"
 TLS_VERSION_DEFAULT_MIN = None
 TLS_VERSION_DEFAULT_MAX = None
 
+DEFAULT_KEY_TYPE_SIZE = "rsa:2048"
+
 SD_IPA_API_MESSAGE_ID = uuid.uuid3(uuid.NAMESPACE_DNS, 'IPA.API')
 
 # regular expression NameSpace member names must match:
@@ -190,8 +191,9 @@ DEFAULT_CONFIG = (
 
     # Time to wait for a service to start, in seconds.
     # Note that systemd has a DefaultTimeoutStartSec of 90 seconds. Higher
-    # values are not effective unless systemd is reconfigured, too.
-    ('startup_timeout', 120),
+    # values are not effective unless systemd is reconfigured, too. Or you
+    # can update the systemd service file with its own TimeoutStartSec.
+    ('startup_timeout', 90),
     # How long http connection should wait for reply [seconds].
     ('http_timeout', 30),
     # How long to wait for an entry to appear on a replica
@@ -251,6 +253,9 @@ DEFAULT_CONFIG = (
     # Ignore TTL. Perform schema call and download schema if not in cache.
     ('force_schema_check', False),
 
+    # Default key type and size for IPA certs (HTTP, DS, KDC and RA)
+    ('key_type_size', DEFAULT_KEY_TYPE_SIZE),
+
     # ********************************************************
     #  The remaining keys are never set from the values here!
     # ********************************************************
@@ -369,6 +374,16 @@ SUBID_RANGE_SIZE = SUBID_RANGE_MAX - SUBID_RANGE_START
 # threshold before DNA plugin requests a new range
 SUBID_DNA_THRESHOLD = 500
 
+# moved from ipaserver/install/cainstance.py::CAInstance to make importing
+# the values easier without pulling in half the installer codebase.
+CA_TRACKING_REQS = {
+    'auditSigningCert cert-pki-ca': 'caSignedLogCert',
+    'ocspSigningCert cert-pki-ca': 'caOCSPCert',
+    'subsystemCert cert-pki-ca': 'caSubsystemCert',
+    'caSigningCert cert-pki-ca': 'caCACert',
+    'Server-Cert cert-pki-ca': 'caServerCert',
+}
+
 # moved from ipaserver/install/krainstance.py::KRAInstance to avoid duplication
 # as per https://pagure.io/freeipa/issue/8795
 KRA_TRACKING_REQS = {
@@ -389,7 +404,6 @@ VAULT_WRAPPING_SUPPORTED_ALGOS = (
 VAULT_WRAPPING_DEFAULT_ALGO = VAULT_WRAPPING_AES128_CBC
 
 # Add 3DES for backwards compatibility if supported
-if getattr(algorithms, 'TripleDES', None):
-    if backend.cipher_supported(algorithms.TripleDES(
-                                b"\x00" * 8), modes.CBC(b"\x00" * 8)):
-        VAULT_WRAPPING_SUPPORTED_ALGOS += (VAULT_WRAPPING_3DES,)
+if backend.cipher_supported(TripleDES(
+                            b"\x00" * 8), modes.CBC(b"\x00" * 8)):
+    VAULT_WRAPPING_SUPPORTED_ALGOS += (VAULT_WRAPPING_3DES,)
