@@ -86,7 +86,10 @@ class Group(_Entity):
             try:
                 self._entity = entity = grp.getgrnam(self)
             except KeyError:
-                raise ValueError(f"group '{self!s}' not found") from None
+                try:
+                    self._entity = entity = grp.getgrgid(int(self))
+                except (TypeError, ValueError):
+                    raise ValueError(f"group '{self!s}' not found") from None
         return entity
 
     @property
@@ -120,7 +123,9 @@ class BaseConstantsNamespace:
     NAMED_DATA_DIR = "data/"
     NAMED_OPTIONS_VAR = "OPTIONS"
     NAMED_OPENSSL_ENGINE = None
+    NAMED_OPENSSL_PROVIDER = None
     NAMED_ZONE_COMMENT = ""
+    NOBODY_GROUP = Group("nobody")
     PKI_USER = User("pkiuser")
     PKI_GROUP = Group("pkiuser")
     # ntpd init variable used for daemon options
