@@ -18,8 +18,11 @@ from ipaplatform.osinfo import osinfo
 # RHEL 8 uses /etc/nfs.conf
 HAS_NFS_CONF = osinfo.version_number >= (8,)
 # RHEL 9 uses pkcs11 as openssl engine
-HAS_PKCS11_OPENSSL_ENGINE = osinfo.version_number >= (9,)
+HAS_PKCS11_OPENSSL_ENGINE = osinfo.version_number == (9,)
 
+# RHEL 10 and later deprecated OpenSSL engine and recommend using OpenSSL
+# provider API.
+HAS_OPENSSL_PROVIDER = osinfo.version_number >= (10,)
 
 __all__ = ("constants", "User", "Group")
 
@@ -31,5 +34,7 @@ class RHELConstantsNamespace(RedHatConstantsNamespace):
         SECURE_NFS_VAR = None
     if HAS_PKCS11_OPENSSL_ENGINE:
         NAMED_OPENSSL_ENGINE = "pkcs11"
+    if HAS_OPENSSL_PROVIDER:
+        NAMED_OPENSSL_PROVIDER = True
 
 constants = RHELConstantsNamespace()
