@@ -17,13 +17,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from pkg_resources import parse_version
+from packaging.version import parse as parse_version
 
 # The full version including strings
-VERSION = "4.12.4"
+VERSION = "4.13.0"
 
 # A fuller version including the vendor tag (e.g. 3.3.3-34.fc20)
-VENDOR_VERSION = "4.12.4"
+VENDOR_VERSION = "4.13.0"
 
 
 # Just the numeric portion of the version so one can do direct numeric
@@ -43,11 +43,11 @@ VENDOR_VERSION = "4.12.4"
 #   IPA 3.2.1:  NUM_VERSION=30201
 #   IPA 3.2.99: NUM_VERSION=30299 (development version)
 #   IPA 3.3.0:  NUM_VERSION=30300
-NUM_VERSION = 41204
+NUM_VERSION = 41300
 
 
 # The version of the API.
-API_VERSION = "2.254"
+API_VERSION = "2.257"
 
 
 DEFAULT_PLUGINS = frozenset(l.strip() for l in """
@@ -577,6 +577,15 @@ sudorule_remove_runasgroup/1
 sudorule_remove_runasuser/1
 sudorule_remove_user/1
 sudorule_show/1
+sysaccount/1
+sysaccount_add/1
+sysaccount_del/1
+sysaccount_disable/1
+sysaccount_enable/1
+sysaccount_find/1
+sysaccount_mod/1
+sysaccount_policy/1
+sysaccount_show/1
 topic/1
 topic_find/1
 topic_show/1
