@@ -411,6 +411,10 @@ class i18n_messages(Command):
             "login_certificate_desc": _("Log in using personal certificate"),
             "logout": _("Log out"),
             "logout_error": _("Log out error"),
+            "new_ui_msg": _(
+                "<i class=\"fa fa-info-circle\"></i> We have launched "
+                "new UI, <a href='${protocol}//${host}/ipa/modern-ui'>"
+                "try it out!</a>"),
             "password": _("Password"),
             "password_and_otp": _("Password or Password+One-Time Password"),
             "redirect_msg": _("You will be redirected in ${count}s"),
@@ -1100,7 +1104,7 @@ class i18n_messages(Command):
             "idp": {
                 "template_keycloak": _("Keycloak or Red Hat SSO"),
                 "template_google": _("Google"),
-                "template_github": _("Github"),
+                "template_github": _("GitHub"),
                 "template_microsoft": _("Microsoft or Azure"),
                 "template_okta": _("Okta"),
                 "label_idpclient": _("OAuth 2.0 client details"),
@@ -1533,6 +1537,24 @@ class i18n_messages(Command):
             "sudooptions": {
                 "remove": _("Remove sudo options"),
             },
+            "sysaccount": {
+                "add": _("Add system account"),
+                "remove": _("Remove system account"),
+                "add_into_roles": _(
+                    "Add system account '${primary_key}' into roles"
+                ),
+                "remove_from_roles": _(
+                    "Remove system account '${primary_key}' from roles"
+                ),
+                "privileged": _(
+                    "Change passwords without reset"
+                ),
+                "privileged_confirm": _(
+                    "Change privileged status of '${primary_key}' "
+                    "system account"
+                )
+
+            },
             "topology": {
                 "autogenerated": _("Autogenerated"),
                 "segment_details": _("Segment details"),
