@@ -55,15 +55,20 @@ class TestPWPolicy(IntegrationTest):
             ),
         )
 
-    def reset_password(self, host, user=USER, password=PASSWORD):
+    def reset_password(self, host, user=USER, password=PASSWORD,
+                       unlock=False):
         tasks.kinit_admin(host)
         host.run_command(
             ['ipa', 'passwd', user],
             stdin_text='{password}\n{password}\n'.format(password=password),
         )
+        if unlock:
+            host.run_command(['ipa', 'user-unlock', user])
+        tasks.kdestroy_all(host)
 
     def set_pwpolicy(self, minlength=None, maxrepeat=None, maxsequence=None,
-                     dictcheck=None, usercheck=None, minclasses=None):
+                     dictcheck=None, usercheck=None, minclasses=None,
+                     dcredit=None, ucredit=None, lcredit=None, ocredit=None):
         tasks.kinit_admin(self.master)
         args = ["ipa", "pwpolicy-mod", POLICY]
         if minlength is not None:
@@ -78,6 +83,14 @@ class TestPWPolicy(IntegrationTest):
             args.append("--usercheck={}".format(usercheck))
         if minclasses is not None:
             args.append("--minclasses={}".format(minclasses))
+        if dcredit is not None:
+            args.append("--dcredit={}".format(dcredit))
+        if ucredit is not None:
+            args.append("--ucredit={}".format(ucredit))
+        if lcredit is not None:
+            args.append("--lcredit={}".format(lcredit))
+        if ocredit is not None:
+            args.append("--ocredit={}".format(ocredit))
         self.master.run_command(args)
 
         self.reset_password(self.master)
@@ -92,7 +105,11 @@ class TestPWPolicy(IntegrationTest):
              "--dictcheck" ,"false",
              "--minlife", "0",
              "--minlength", "0",
-             "--minclasses", "0",],
+             "--minclasses", "0",
+             "--dcredit", "0",
+             "--ucredit", "0",
+             "--lcredit", "0",
+             "--ocredit", "0",],
         )
         # minlength => 6 is required for any of the libpwquality settings
         self.master.run_command(
@@ -226,6 +243,89 @@ class TestPWPolicy(IntegrationTest):
             self.kinit_as_user(self.master, PASSWORD, valid)
             self.reset_password(self.master)
 
+    def test_credits(self, reset_pwpolicy):
+        """Test each credit individually. This would be all copy/paste
+           if done individually so provide a config and set of good
+           and bad passwords to test.
+
+           Reminder: a positive credit reduces minlength
+                     a zero credit is a no-op
+                     a negative credit is the minimum required
+        """
+        def test_policy(good, bad, msg, **options):
+            """Validate the configuration.
+
+               Given the options in **options, the other arguments are:
+               good: tuple of valid passwords
+               bad: tuple of invalid passwords
+               msg: tuple of expected messages. LDAP may return a different
+                    one than kinit.
+            """
+
+            tasks.kinit_admin(self.master)
+            self.clean_pwpolicy()
+            self.set_pwpolicy(**options)
+
+            for password in good:
+                self.kinit_as_user(self.master, PASSWORD, password)
+                self.reset_password(self.master)
+
+            for password in bad:
+                result = self.kinit_as_user(self.master, PASSWORD, password,
+                                            raiseonerr=False)
+                assert result.returncode == 1
+                for message in msg:
+                    if message in result.stdout_text:
+                        break
+                else:
+                    assert False, "%s not in %s" % (
+                        result.stdout_text, ', '.join(msg)
+                    )
+                result = tasks.ldappasswd_user_change(USER, PASSWORD, password,
+                                                      self.master,
+                                                      raiseonerr=False)
+                assert result.returncode == 1
+                for message in msg:
+                    if message in result.stdout_text:
+                        break
+                else:
+                    assert False, "%s not in %s" % (
+                        result.stdout_text, ', '.join(msg)
+                    )
+
+        # hint: minlength == 6
+        # dcredit
+        test_policy(('Pass12', 'password1'), ('123', 'passw'),
+                    ('Password is too short',), **{'dcredit': 1})
+        test_policy(('Passw0rd1', 'password12'), ('Passwd1', 'password1'),
+                    ('Password does not contain enough character',
+                     'Password contains too few digits',),
+                    **{'dcredit': -2})
+
+        # ucredit
+        test_policy(('Passw', 'PASSWORD'), ('PAS', 'passw'),
+                    ('Password is too short',), **{'ucredit': 1})
+        test_policy(('Passw0rD1', 'PasswordS'), ('Passwd1', 'password1'),
+                    ('Password does not contain enough character',
+                     'Password contains too few upper characters',),
+                    **{'ucredit': -2})
+
+        # lcredit
+        test_policy(('PASSw', 'password'), ('pas', 'PASSW'),
+                    ('Password is too short',), **{'lcredit': 1})
+        test_policy(('Passw0rD1', 'PasswordS'), ('PASSWd1', 'PASSWoRD1'),
+                    ('Password does not contain enough character',
+                     'Password contains too few lower characters',),
+                    **{'lcredit': -2})
+
+        # ocredit
+        test_policy(('passw!', 'password#'), ('pas#', 'PAS$'),
+                    ('Password is too short',), **{'ocredit': 1})
+        test_policy(('Passw@rD1$', 'P&sswordS%'), ('passwd*', 'password^'),
+                    ('Password does not contain enough character',
+                     'Password contains too few special characters',),
+                    **{'ocredit': -2})
+
     def test_minlength_mod(self, reset_pwpolicy):
         """Test that the pwpolicy minlength overrides our policy
         """
@@ -245,7 +345,7 @@ class TestPWPolicy(IntegrationTest):
             assert result.returncode != 0
             assert 'minlength' in result.stderr_text
 
-        # With any pwq value set, setting minlife < 6 should fail
+        # With any pwq value set, setting minlength < 6 should fail
         for values in (('--maxrepeat', '4'),
                        ('--maxsequence', '4'),
                        ('--dictcheck', 'true'),
@@ -388,6 +488,7 @@ class TestPWPolicy(IntegrationTest):
         dn = "uid={user},cn=users,cn=accounts,{base_dn}".format(
              user=USER, base_dn=str(self.master.domain.basedn))
 
+        tasks.kinit_admin(self.master)
         self.master.run_command(
             ["ipa", "pwpolicy-mod", POLICY, "--gracelimit", "0", ],
         )
